==> go-ld-prime-v0.0.1.tar.gz: go-ld-prime-v0.0.1/codec/dagjson/roundtripCidlink_test.go <==
package dagjson

import (
	"bytes"
	"io"
	"strings"
	"testing"

	. "github.com/warpfork/go-wish"

	cid "gitlab.dms3.io/dms3/public/go-cid"
	ld "gitlab.dms3.io/ld/go-ld-prime"
	cidlink "gitlab.dms3.io/ld/go-ld-prime/linking/cid"
	basicnode "gitlab.dms3.io/ld/go-ld-prime/node/basic"
)

func TestRoundtripCidlink(t *testing.T) {
	lp := cidlink.LinkPrototype{cid.Prefix{
		Version:  1,
		Codec:    0x0129,
		MhType:   0x13,
		MhLength: 4,
	}}
	lsys := cidlink.DefaultLinkSystem()

	buf := bytes.Buffer{}
	lsys.StorageWriteOpener = func(lnkCtx ld.LinkContext) (io.Writer, ld.BlockWriteCommitter, error) {
		return &buf, func(lnk ld.Link) error { return nil }, nil
	}
	lsys.StorageReadOpener = func(lnkCtx ld.LinkContext, lnk ld.Link) (io.Reader, error) {
		return bytes.NewReader(buf.Bytes()), nil
	}

	lnk, err := lsys.Store(ld.LinkContext{}, lp, n)
	Require(t, err, ShouldEqual, nil)

	n2, err := lsys.Load(ld.LinkContext{}, lnk, basicnode.Prototype.Any)
	Require(t, err, ShouldEqual, nil)
	Wish(t, n2, ShouldEqual, nSorted)
}

// Make sure that a map that *almost* looks like a link is handled safely.
//
// This is aiming very specifically at the corner case where a minimal number of
// tokens have to be reprocessed before a recursion that find a real link appears.
func TestUnmarshalTrickyMapContainingLink(t *testing.T) {
	// Create a link; don't particularly care about its contents.
	lnk := cidlink.LinkPrototype{cid.Prefix{
		Version:  1,
		Codec:    0x71,
		MhType:   0x13,
		MhLength: 4,
	}}.BuildLink([]byte{1, 2, 3, 4}) // dummy value, content does not matter to this test.

	// Compose the tricky corpus.  (lnk.String "happens" to work here, although this isn't recommended or correct in general.)
	tricky := `{"/":{"/":"` + lnk.String() + `"}}`

	// Unmarshal.  Hopefully we get a map with a link in it.
	nb := basicnode.Prototype__Any{}.NewBuilder()
	err := Decode(nb, strings.NewReader(tricky))
	Require(t, err, ShouldEqual, nil)
	n := nb.Build()
	Wish(t, n.Kind(), ShouldEqual, ld.Kind_Map)
	n2, err := n.LookupByString("/")
	Require(t, err, ShouldEqual, nil)
	Wish(t, n2.Kind(), ShouldEqual, ld.Kind_Link)
}

==> go-ld-prime-v0.0.2.tar.gz: go-ld-prime-v0.0.2/codec/dagjson/roundtripCidlink_test.go <==
package dagjson

import (
	"bytes"
	"io"
	"strings"
	"testing"

	. "github.com/warpfork/go-wish"

	cid "gitlab.dms3.io/dms3/go-cid"
	ld "gitlab.dms3.io/ld/go-ld-prime"
	cidlink "gitlab.dms3.io/ld/go-ld-prime/linking/cid"
	basicnode "gitlab.dms3.io/ld/go-ld-prime/node/basic"
)

func TestRoundtripCidlink(t *testing.T) {
	lp := cidlink.LinkPrototype{cid.Prefix{
		Version:  1,
		Codec:    0x0129,
		MhType:   0x13,
		MhLength: 4,
	}}
	lsys := cidlink.DefaultLinkSystem()

	buf := bytes.Buffer{}
	lsys.StorageWriteOpener = func(lnkCtx ld.LinkContext) (io.Writer, ld.BlockWriteCommitter, error) {
		return &buf, func(lnk ld.Link) error { return nil }, nil
	}
	lsys.StorageReadOpener = func(lnkCtx ld.LinkContext, lnk ld.Link) (io.Reader, error) {
		return bytes.NewReader(buf.Bytes()), nil
	}

	lnk, err := lsys.Store(ld.LinkContext{}, lp, n)
	Require(t, err, ShouldEqual, nil)

	n2, err := lsys.Load(ld.LinkContext{}, lnk, basicnode.Prototype.Any)
	Require(t, err, ShouldEqual, nil)
	Wish(t, n2, ShouldEqual, nSorted)
}

// Make sure that a map that *almost* looks like a link is handled safely.
//
// This is aiming very specifically at the corner case where a minimal number of
// tokens have to be reprocessed before a recursion that find a real link appears.
func TestUnmarshalTrickyMapContainingLink(t *testing.T) {
	// Create a link; don't particularly care about its contents.
	lnk := cidlink.LinkPrototype{cid.Prefix{
		Version:  1,
		Codec:    0x71,
		MhType:   0x13,
		MhLength: 4,
	}}.BuildLink([]byte{1, 2, 3, 4}) // dummy value, content does not matter to this test.

	// Compose the tricky corpus.  (lnk.String "happens" to work here, although this isn't recommended or correct in general.)
	tricky := `{"/":{"/":"` + lnk.String() + `"}}`

	// Unmarshal.  Hopefully we get a map with a link in it.
	nb := basicnode.Prototype__Any{}.NewBuilder()
	err := Decode(nb, strings.NewReader(tricky))
	Require(t, err, ShouldEqual, nil)
	n := nb.Build()
	Wish(t, n.Kind(), ShouldEqual, ld.Kind_Map)
	n2, err := n.LookupByString("/")
	Require(t, err, ShouldEqual, nil)
	Wish(t, n2.Kind(), ShouldEqual, ld.Kind_Link)
}
